
//						
// INTRODUCTION
//    This is a simple TCP socket program for the Lambda Genesys power supply with LAN.
//    It allows you to, as fast as possible, send many SCPI commands and queries.
//		Author:   Walter Heyck, TDK-Lambda, Sept 26, 2008

//      Version: "GEN TCP Socket C# CmdArray v03"    Aug 12, 2009
//      Makes the socket BLOCKING so that bad queries, which return a TIMEOUT error,
//		  do not cascade into errors for subsequent queries.
//		Formats the output so "Send" and "Receive" lines up nicely.
//		Adds prompt after error to continue or quit.

// SETUP:
//    This program requires the Visual Studio 2005 Edit/Debug IDE.
//    Edit the "cmdStrs" array list of SCPI commands and querires to send.
//       You may add, delete or modify the array elements with no problem.
//    Then click RUN (or F5 key).
//    You will see a console window open. The list of commands and responses will
//       scroll down automatically for up to the "CMDS_TO_SEND" value.
// USAGE:
//    This program runs automatically after RUN is clicked.
//    If an error occurs, it will show an exception box and then end.
// NOTE:
//	  This program opens TCP socket port 8003.
//



using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;


namespace TCPSocketCSCmdArray
{
	class Program
	{

		static void Main( string[ ] args )
		{
			// This is default IP addr, change it for your supply
			IPAddress targetIP = IPAddress.Parse( "169.254.252.145" );

			// Number of SCPI commands to send before quitting 
			const int CMDS_TO_SEND = 100;

			// Timeout value required of program locks up if there is never a response
			const int RCV_TIME_OUT = 2000;   // in milliseconds

			// REMEMBER: 
			// 1. Only 4 commands may be sent before a query must be sent.
			//    This prevent the controller from sending too many commands and causing
			//      the Genesys LAN input buffer to overflow and possibly crash.
			// 2. The MEAS:VOLT? or MEAS:CURR? may return unexpected measurements
			//      because there are no time delays to allow the output to settle
			//      and to allow the ADC to make their conversions.

			string[ ] cmdStrs = new string[ ]
			{ 
				":VOLT 3.3;",
				":CURR 10.0;",
				"OUTP:STAT ON;",
				"SYST:ERR?;",
				":CURR:PROT:STAT OFF;",
				":VOLT:PROT:LEV MAX;",
				"MEAS:VOLT?",
				"MEAS:CURR?",
				":VOLT 4.4;",
				":VOLT:LIM:LOW 0.0;",
				"OUTP:PON ON;",
				"SYST:ERR?;"
			};
			// Other commands
				//"SYST:ERR?",
				//"MEAS:VOLT?",
				//"MEAS:CURR?",
				//"SOUR:MOD?",
				//"*ESR?"

			string cmdLine;							// SCPI command from cmdStrs array
			byte[ ] recvBytes = new byte[ 256 ]; 	// Buffer to receive socket responses
			int sendCount = 0;						// Bytes sent during socket Send
			int recvCount = 0;						// Bytes received during socket Receive
			int sentQty = 0;						// Number of queries sent, up to 1000
			ConsoleKeyInfo inKey;					// Key press to continue after error

			Console.WriteLine( "\nGenesys - TCP socket \"Fast Command\" program v03." );
			Console.WriteLine( "-- Use command \"Ctrl-C\" to exit," );
			Console.WriteLine( "--   or wait {0} until commands sent.\n\n", Convert.ToString( CMDS_TO_SEND) );

			Console.WriteLine( "Enter IP address" );
			Console.Write( "(default {0}) --> ", Convert.ToString( targetIP) );
			cmdLine = Console.ReadLine( );

			if ( cmdLine.Length > 6 )
				targetIP = IPAddress.Parse( cmdLine );


			Socket mySocket = new Socket( AddressFamily.InterNetwork,
											SocketType.Stream,
											ProtocolType.Tcp );
			mySocket.ReceiveTimeout = RCV_TIME_OUT;

			try
			{
				mySocket.Connect( new IPEndPoint( targetIP, 8003 ) );
			}
			catch ( SocketException e )
			{
				Console.WriteLine( "\"Connect error: \" {0} Error code: {1}.",
												e.Message, e.ErrorCode );
				Console.WriteLine( "\nPress <Enter> to close console and quit." );
				cmdLine = Console.ReadLine( );
				return;
			}

			// Loop over each string in SCPI array

			while ( sentQty <= CMDS_TO_SEND )
			{
				foreach ( string cmd in cmdStrs )
				{
					// Convert the string into a byte array
					byte[ ] cmdBytes = Encoding.ASCII.GetBytes( cmd );

					// Increment SCPI sent counter
					if ( ++sentQty > CMDS_TO_SEND )
						break;

					recvCount = 0;
					Array.Clear( recvBytes, 0, 256 );

					try
					{
						// Blocks until Send returns.
						sendCount = mySocket.Send( cmdBytes,
											cmdBytes.Length, SocketFlags.None );

						Console.WriteLine( "{0}: Send {1} chars = \"{2}\"", 
							Convert.ToString( sentQty).PadLeft( 4),
							Convert.ToString( sendCount).PadLeft( 2), 
							cmd );

						// If SCPI string has "?", then is query and need to read response
						if ( cmd.IndexOf( "?" ) >= 0 )
						{
							// Added to fix problem where a bad query would timeout and the 
							//   next query would return another error 10035 and all queries there after
							//   would respond with the message from the query before it
							mySocket.Blocking = true;

							// Is query, so read the response
							recvCount = mySocket.Receive( recvBytes, SocketFlags.None );

							Console.WriteLine( "      Recv {0} chars   \"{1}\"", 
								Convert.ToString( recvCount).PadLeft( 2),
								Encoding.ASCII.GetString( recvBytes ).Substring( 0,recvCount));
						}  // End If
					}
					catch ( SocketException e )
					{
						Console.WriteLine( "ERROR: \n{0}. Error code: {1}.", e.Message, e.ErrorCode );
						Console.WriteLine( "<Enter> = Quit.  <Any Key> = Continue.\n" );

						inKey = Console.ReadKey( true);
						if ( inKey.Key == ConsoleKey.Enter)
							return;			// Quit program if <Enter> pressed
					}
				}		// End ForEach loop
			}       //  End While loop

			mySocket.Close( );

			Console.WriteLine( "\nPress <Enter> to close console and quit." );
			cmdLine = Console.ReadLine( );
		}
	}
}
